@@ -45,8 +45,7 @@ struct _GVirInputStreamPrivate
     GVirStream *stream;
 
     /* pending operation metadata */
-    GSimpleAsyncResult *result;
-    GCancellable *cancellable;
+    GTask *task;
     gpointer buffer;
     gsize count;
 };
@@ -89,7 +88,6 @@ static void gvir_input_stream_finalize(GObject *object)
 {
     GVirInputStream *stream = GVIR_INPUT_STREAM(object);
 
-    g_debug("Finalize input stream GVirStream=%p", stream->priv->stream);
     stream->priv->stream = NULL; // unowned
 
     if (G_OBJECT_CLASS(gvir_input_stream_parent_class)->finalize)
@@ -103,48 +101,45 @@ gvir_input_stream_read_ready(GVirStream *stream,
 {
     GVirInputStream *input_stream = GVIR_INPUT_STREAM(opaque);
     GVirInputStreamPrivate *priv = input_stream->priv;
-    GSimpleAsyncResult *simple = priv->result;
+    GTask *task = priv->task;
+    GCancellable *cancellable = g_task_get_cancellable(task);
     GError *error = NULL;
     gssize result;
 
+    priv->task = NULL;
+
     if (!(cond & GVIR_STREAM_IO_CONDITION_READABLE)) {
         g_warn_if_reached();
-        g_simple_async_result_set_error(simple,
-                                        G_IO_ERROR,
-                                        G_IO_ERROR_INVALID_ARGUMENT,
-                                        "%s",
-                                        "Expected stream to be readable");
+        g_task_return_new_error(task,
+                                G_IO_ERROR,
+                                G_IO_ERROR_INVALID_ARGUMENT,
+                                "%s",
+                                "Expected stream to be readable");
         goto cleanup;
     }
 
-    result  = gvir_stream_receive(stream, priv->buffer, priv->count,
-                                  priv->cancellable, &error);
+    result = gvir_stream_receive(stream, priv->buffer, priv->count,
+                                 cancellable, &error);
+    if (error != NULL) {
+        if (g_error_matches(error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
+            g_warn_if_reached();
+            g_task_return_new_error(task,
+                                    G_IO_ERROR,
+                                    G_IO_ERROR_INVALID_ARGUMENT,
+                                    "%s",
+                                    "Expected stream to be readable");
+            g_error_free (error);
+        } else {
+            g_task_return_error(task, error);
+        }
 
-    if (g_error_matches(error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
-        g_warn_if_reached();
-        g_simple_async_result_set_error(simple,
-                                        G_IO_ERROR,
-                                        G_IO_ERROR_INVALID_ARGUMENT,
-                                        "%s",
-                                        "Expected stream to be readable");
         goto cleanup;
     }
 
-    if (result >= 0)
-        g_simple_async_result_set_op_res_gssize(simple, result);
-
-    if (error)
-        g_simple_async_result_take_error(simple, error);
-
-    if (priv->cancellable) {
-        g_object_unref(priv->cancellable);
-        priv->cancellable = NULL;
-    }
+    g_task_return_int(task, result);
 
 cleanup:
-    priv->result = NULL;
-    g_simple_async_result_complete(simple);
-    g_object_unref(simple);
+    g_object_unref(task);
     return FALSE;
 }
 
@@ -159,7 +154,7 @@ static void gvir_input_stream_read_async(GInputStream *stream,
     GVirInputStream *input_stream = GVIR_INPUT_STREAM(stream);
 
     g_return_if_fail(GVIR_IS_INPUT_STREAM(stream));
-    g_return_if_fail(input_stream->priv->result == NULL);
+    g_return_if_fail(input_stream->priv->task == NULL);
 
     gvir_stream_add_watch_full(input_stream->priv->stream,
                                G_PRIORITY_DEFAULT,
@@ -168,12 +163,8 @@ static void gvir_input_stream_read_async(GInputStream *stream,
                                g_object_ref(stream),
                                (GDestroyNotify)g_object_unref);
 
-    input_stream->priv->result =
-        g_simple_async_result_new(G_OBJECT(stream), callback, user_data,
-                                  gvir_input_stream_read_async);
-    if (cancellable)
-        g_object_ref(cancellable);
-    input_stream->priv->cancellable = cancellable;
+    input_stream->priv->task =
+        g_task_new(stream, cancellable, callback, user_data);
     input_stream->priv->buffer = buffer;
     input_stream->priv->count = count;
 }
@@ -181,22 +172,18 @@ static void gvir_input_stream_read_async(GInputStream *stream,
 
 static gssize gvir_input_stream_read_finish(GInputStream *stream,
                                             GAsyncResult *result,
-                                            GError **error G_GNUC_UNUSED)
+                                            GError **error)
 {
     GVirInputStream *input_stream = GVIR_INPUT_STREAM(stream);
-    GSimpleAsyncResult *simple;
     virStreamPtr handle;
     gssize count;
 
     g_return_val_if_fail(GVIR_IS_INPUT_STREAM(stream), -1);
-    g_return_val_if_fail(g_simple_async_result_is_valid(result, G_OBJECT(stream),
-                                                        gvir_input_stream_read_async),
-                         -1);
+    g_return_val_if_fail(g_task_is_valid(result, stream), -1);
+    g_return_val_if_fail(error == NULL || *error == NULL, -1);
     g_object_get(input_stream->priv->stream, "handle", &handle, NULL);
 
-    simple = G_SIMPLE_ASYNC_RESULT(result);
-
-    count = g_simple_async_result_get_op_res_gssize(simple);
+    count = g_task_propagate_int(G_TASK(result), error);
 
     virStreamFree(handle);
 
